/*
 * (C) Copyright 2011
 * CompuLab Ltd. <www.compulab.co.il>
 * Igor Grinberg <grinberg@compulab.co.il>
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc.
 */

#include <common.h>
#include <asm/arch/bits.h>
#include <asm/arch/mux.h>
#include <asm/arch/sys_proto.h>
#include <asm/arch/sys_info.h>
#include <asm/arch/mem.h>

#include "common.h"
#include "prcm.h"
#include "memory.h"
#include "mux.h"
#include "board.h"

/*
 * DPLL(1-3) PARAM tables for 36x/37x
 * Each of the tables has (ordered) M, N, FREQSEL, M2 values.
 * Populated for 13MHz only.
 */
static struct dpll_param cm_t3730_mpu_dpll_param[] = {
/*	M	N	fsel	M2 */
	{500,	12,	0,	1},
	{500,	12,	0,	1},
	{500,	12,	0,	1},
	{500,	12,	0,	1},
};

static struct dpll_param cm_t3730_iva_dpll_param[] = {
/*	 M	N	fsel	M2 */
	{10,	0,	0,	1},
	{10,	0,	0,	1},
	{10,	0,	0,	1},
	{10,	0,	0,	1},
};

static struct dpll_param cm_t3730_core_dpll_param[] = {
/*	M	N	fsel	M2 */
	{332,	12,	0,	2},
	{332,	12,	0,	2},
	{332,	12,	0,	2},
	{332,	12,	0,	2},
};

static struct dpll_per_36x_param cm_t3730_per_dpll_param[] = {
/*	SYSCLK	M	N	M2	M3	M4	M5	M6	m2DIV */
	{26000,	432,	12,	9,	16,	9,	4,	3,	1},
	{26000,	432,	12,	9,	16,	9,	4,	3,	1},
	{26000,	432,	12,	9,	16,	9,	4,	3,	1},
	{26000,	432,	12,	9,	16,	9,	4,	3,	1},
};

/* NAND GPMC config table */
static u32 cm_t3730_nand_gpmc_config[GPMC_MAX_REG] = {
	SMNAND_GPMC_CONFIG1,
	SMNAND_GPMC_CONFIG2,
	SMNAND_GPMC_CONFIG3,
	SMNAND_GPMC_CONFIG4,
	SMNAND_GPMC_CONFIG5,
	SMNAND_GPMC_CONFIG6,
	0
};

static inline int cpu_is_omap36xx(void)
{
	return is_cpu_family() == CPU_OMAP36XX;
}

int cpu_is_omap37xx(void)
{
	return cpu_is_omap36xx();
}

static void cm_t3730_set_muxconf_regs(void)
{
	set_sdrc_muxconf_regs();
	set_gpmc_muxconf_regs();
	set_jtag_muxconf_regs();

	/* CM-T3730 specific mux settings */
	MUX_VAL(CP(GPMC_nCS0),		(IDIS | PTU | EN  | M0));
	MUX_VAL(CP(GPMC_WAIT0),		(IEN  | PTU | EN  | M0));
	MUX_VAL(CP(GPMC_WAIT1),		(IEN  | PTU | EN  | M0));
	MUX_VAL(CP(UART3_TX_IRTX),	(IDIS | PTD | DIS | M0)); /*UART3_TX*/
	MUX_VAL(CP(UART3_RX_IRRX),	(IEN  | PTD | DIS | M0)); /*UART3_RX*/
	MUX_VAL(CP(SYS_32K),		(IEN  | PTD | DIS | M0));
	MUX_VAL(CP(SYS_BOOT2),		(IEN  | PTD | DIS | M4)); /*GPIO4*/
	MUX_VAL(CP(SYS_CLKOUT2),	(IDIS | PTD | DIS | M4)); /*GPIO186*/
	MUX_VAL(CP(sdrc_cke0),		(IDIS | PTU | EN  | M0));
	MUX_VAL(CP(sdrc_cke1),		(IDIS | PTD | DIS | M7)); /*not used*/
}

static void cm_t3730_set_board_data(struct board_data *data)
{
	if (!data)
		hang();

	data->board_name		= "CM-T3730";
	data->watchdog_init		= watchdog_init;
	data->try_unlock_memory		= try_unlock_memory;
	data->set_muxconf_regs		= cm_t3730_set_muxconf_regs;
	data->prcm_init			= prcm_init;
	data->per_clocks_enable		= per_clocks_enable;
	data->setup_memory_controller	= setup_sdrc_memory_controller;
	data->core_dpll_param		= cm_t3730_core_dpll_param;
	data->per_dpll_param		= cm_t3730_per_dpll_param;
	data->mpu_dpll_param		= cm_t3730_mpu_dpll_param;
	data->iva_dpll_param		= cm_t3730_iva_dpll_param;
	data->nand_gpmc_config		= cm_t3730_nand_gpmc_config;
}

int cm_t3730_probe(struct board_data *data)
{
	if (!cpu_is_omap37xx())
		return 1;

	cm_t3730_set_board_data(data);
	return 0;
}

